@@ -1,8 +1,9 @@
-import py, sys, os
+import sys, os
 import subprocess, weakref
+import pytest
 from cffi import FFI
 from cffi.backend_ctypes import CTypesBackend
-from testing.support import u
+from testing.support import u, is_musl
 
 
 SOURCE = """\
@@ -35,6 +36,10 @@ typedef struct {
     long bottom;
 } RECT;
 
+typedef struct {
+    unsigned char a, b, c;
+} THREEBYTES;
+
 
 EXPORT int PointInRect(RECT *prc, POINT pt)
 {
@@ -107,6 +112,15 @@ EXPORT void modify_struct_value(RECT r)
 {
     r.left = r.right = r.top = r.bottom = 500;
 }
+
+EXPORT THREEBYTES return_three_bytes(void)
+{
+    THREEBYTES result;
+    result.a = 12;
+    result.b = 34;
+    result.c = 56;
+    return result;
+}
 """
 
 class TestOwnLib(object):
@@ -167,9 +181,9 @@ class TestOwnLib(object):
 
     def test_getting_errno(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         if sys.platform == 'win32':
-            py.test.skip("fails, errno at multiple addresses")
+            pytest.skip("fails, errno at multiple addresses")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             int test_getting_errno(void);
@@ -181,11 +195,11 @@ class TestOwnLib(object):
 
     def test_setting_errno(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         if sys.platform == 'win32':
-            py.test.skip("fails, errno at multiple addresses")
+            pytest.skip("fails, errno at multiple addresses")
         if self.Backend is CTypesBackend and '__pypy__' in sys.modules:
-            py.test.skip("XXX errno issue with ctypes on pypy?")
+            pytest.skip("XXX errno issue with ctypes on pypy?")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             int test_setting_errno(void);
@@ -198,7 +212,7 @@ class TestOwnLib(object):
 
     def test_my_array_7(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             extern int my_array[7];
@@ -208,7 +222,7 @@ class TestOwnLib(object):
             assert ownlib.my_array[i] == i
         assert len(ownlib.my_array) == 7
         if self.Backend is CTypesBackend:
-            py.test.skip("not supported by the ctypes backend")
+            pytest.skip("not supported by the ctypes backend")
         ownlib.my_array = list(range(10, 17))
         for i in range(7):
             assert ownlib.my_array[i] == 10 + i
@@ -218,9 +232,9 @@ class TestOwnLib(object):
 
     def test_my_array_no_length(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         if self.Backend is CTypesBackend:
-            py.test.skip("not supported by the ctypes backend")
+            pytest.skip("not supported by the ctypes backend")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             extern int my_array[];
@@ -228,7 +242,7 @@ class TestOwnLib(object):
         ownlib = ffi.dlopen(self.module)
         for i in range(7):
             assert ownlib.my_array[i] == i
-        py.test.raises(TypeError, len, ownlib.my_array)
+        pytest.raises(TypeError, len, ownlib.my_array)
         ownlib.my_array = list(range(10, 17))
         for i in range(7):
             assert ownlib.my_array[i] == 10 + i
@@ -238,7 +252,7 @@ class TestOwnLib(object):
 
     def test_keepalive_lib(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             int test_getting_errno(void);
@@ -256,7 +270,7 @@ class TestOwnLib(object):
 
     def test_keepalive_ffi(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             int test_getting_errno(void);
@@ -276,7 +290,7 @@ class TestOwnLib(object):
 
     def test_struct_by_value(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             typedef struct {
@@ -317,9 +331,9 @@ class TestOwnLib(object):
 
     def test_addressof_lib(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         if self.Backend is CTypesBackend:
-            py.test.skip("not implemented with the ctypes backend")
+            pytest.skip("not implemented with the ctypes backend")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("extern long left; int test_getting_errno(void);")
         lib = ffi.dlopen(self.module)
@@ -335,9 +349,9 @@ class TestOwnLib(object):
 
     def test_char16_char32_t(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         if self.Backend is CTypesBackend:
-            py.test.skip("not implemented with the ctypes backend")
+            pytest.skip("not implemented with the ctypes backend")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             char16_t foo_2bytes(char16_t);
@@ -350,9 +364,9 @@ class TestOwnLib(object):
 
     def test_modify_struct_value(self):
         if self.module is None:
-            py.test.skip("fix the auto-generation of the tiny test lib")
+            pytest.skip("fix the auto-generation of the tiny test lib")
         if self.Backend is CTypesBackend:
-            py.test.skip("fails with the ctypes backend on some architectures")
+            pytest.skip("fails with the ctypes backend on some architectures")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""
             typedef struct {
@@ -371,3 +385,48 @@ class TestOwnLib(object):
         assert s.top == 22
         assert s.right == 33
         assert s.bottom == 44
+
+    def test_dlopen_handle(self):
+        if self.module is None:
+            pytest.skip("fix the auto-generation of the tiny test lib")
+        if sys.platform == 'win32' or is_musl:
+            pytest.skip("uses 'dl' explicitly")
+        if self.__class__.Backend is CTypesBackend:
+            pytest.skip("not for the ctypes backend")
+        backend = self.Backend()
+        ffi1 = FFI(backend=backend)
+        ffi1.cdef("""void *dlopen(const char *filename, int flags);
+                     int dlclose(void *handle);""")
+        lib1 = ffi1.dlopen('dl')
+        handle = lib1.dlopen(self.module.encode(sys.getfilesystemencoding()),
+                             backend.RTLD_LAZY)
+        assert ffi1.typeof(handle) == ffi1.typeof("void *")
+        assert handle
+
+        ffi = FFI(backend=backend)
+        ffi.cdef("""unsigned short foo_2bytes(unsigned short a);""")
+        lib = ffi.dlopen(handle)
+        x = lib.foo_2bytes(1000)
+        assert x == 1042
+
+        err = lib1.dlclose(handle)
+        assert err == 0
+
+    def test_return_three_bytes(self):
+        if self.module is None:
+            pytest.skip("fix the auto-generation of the tiny test lib")
+        if self.__class__.Backend is CTypesBackend:
+            pytest.skip("not working on win32 on the ctypes backend")
+        ffi = FFI(backend=self.Backend())
+        ffi.cdef("""
+            typedef struct {
+                unsigned char a, b, c;
+            } THREEBYTES;
+
+            THREEBYTES return_three_bytes(void);
+        """)
+        lib = ffi.dlopen(self.module)
+        tb = lib.return_three_bytes()
+        assert tb.a == 12
+        assert tb.b == 34
+        assert tb.c == 56
